        ;************************************
        ;   SOFTWARE FOR ET-BOARD  VER 3.5  *
        ;       PROGRAM TEST DAC0832        *
        ;  ANALOG  OUTPUT OF ET-ADIO BOARD  *
        ;************************************

        CPU "Z80.TBL"
        HOF  "INT8"

        ORG 2000H

IC_DAC: EQU 88H
DSP:    EQU 3FE7H    ; buffer display (7 segment)
BUF:    EQU 3FEFH    ; buffer display (BCD)
HBUF:   EQU 3FF2H    ; buffer (HEX) of HTOD,DTOH
DBUF:   EQU 3FF4H    ; buffer (DEC) of HTOD,DTOH


        ;********************
        ;*   MAIN PROGRAM   *
        ;********************

START:  LD IX,3000H        ; buffer BUF
        LD (IX+0),00H
LOOP:   LD A,(IX+0)        ; set char display
        CALL INDAC
        JR LOOP


        ;***********************
        ;*** INPUT DATA SUB. ***
        ;***********************

INDAC:  LD (BUF+1),A      ; input BCD
        LD A,07H          ; call "UNPK"
        RST 10H           ; BUF -> DSP
        LD A,(DSP+0)
        OR 80H
        LD (DSP+0),A      ; set dot on
        LD HL,00H
        LD (DSP+2),HL     ; off DSP 3,4
        LD H,5EH          ; set char d
        LD L,40H          ; set char -
        LD (DSP+4),HL     ; on DSP 5,6
INDAC1: LD A,05H          ; call "SCANK"
        RST 10H           ; scan DSP & KEY
        LD A,(3FFBH)      ; key code
        CP 13H            ; check Enter key
        JR Z,INDAC2       ; if Enter jump
        CP 0AH            ; check 0-9 only
        JR NC,INDAC1      ; key > 9 new start
        LD HL,DSP         ; save DSP
        CALL BEEP         ; sound beep
        CALL GET2         ; check 2 input
        LD A,(DSP+0)
        OR 80H
        LD (DSP+0),A      ; set dot on
        LD A,0FH          ; call delay
        LD B,80H          ; delay time
        RST 10H
        JR INDAC1         ; loop continue
INDAC2: CALL BEEP         ; sound beep
        LD A,(DSP+0)
        AND 7FH
        LD (DSP+0),A      ; clear dot off
        LD A,06H          ; call "PACK"
        RST 10H           ; DSP -> BUF
        LD A,(BUF+1)
        LD (IX+0),A       ; save BCD
        CP 51H            ; check over 50
        JR C,INDAC3       ; BCD = 0-50 jump
        LD A,(OLDV)       ; volt last value
        CALL UPASS        ; UnPASS display
        JR OUTDAC
INDAC3: LD (DBUF+0),A     ; DEC code
        LD A,0BH          ; call "D TO H"
        RST 10H           ; convert DEC -> HEX
        LD A,(HBUF+0)     ; HEX code
        LD HL,OUTTAB      ; start table
        LD B,00H
        LD C,A
        ADD HL,BC
        LD A,(HL)
        CALL PASS         ; PASS display
OUTDAC: OUT (IC_DAC),A    ; out data to DAC
        LD (OLDV),A       ; save data
        RET


        ;************************
        ;*** GET 2-DIGIT SUB. ***
        ;************************

GET2:   PUSH HL         ; save BCD
UNPKT:  LD HL,SEGTAB    ; 7 segment code
        ADD A,L
        LD L,A
        LD A,0
        ADC A,H
        LD H,A
        LD A,(HL)
CKDGT:  POP HL           ; restore BCD
        LD D,A
        INC HL
        LD A,(HL)
        CP 00H
        JR Z,GET21
        LD (HL),00H
        DEC HL
        SCF              ; set carry flag
GET21:  LD (HL),D
        RET


        ;***********************
        ;*** BEEP SOUND SUB. ***
        ;***********************

BEEP:   PUSH AF
        PUSH BC
        PUSH HL
        LD A,0EH         ; call "HBEEP"
        RST 10H
        POP HL
        POP BC
        POP AF
        RET


        ;***************************
        ;*** -PASS- DISPLAY SUB. ***
        ;***************************

PASS:   PUSH AF
        PUSH BC
        PUSH HL
        LD DE,PASTAB
        LD BC,5F06H      ;
        LD A,0CH         ; call "PRINT"
        RST 10H          ; display -PASS-
        LD A,05H         ; call "SCANK"
        RST 10H          ; scan DSP
        POP HL
        POP BC
        POP AF
        RET


        ;***************************
        ;*** UNPASS DISPLAY SUB. ***
        ;***************************

UPASS:  PUSH AF
        PUSH BC
        PUSH HL
        LD DE,UPATAB
        LD BC,5F06H
        LD A,0CH         ; call "PRINT"
        RST 10H          ; display UnPASS
        LD A,05H         ; call "SCANK"
        RST 10H          ; scan DSP
        POP HL
        POP BC
        POP AF
        RET


PASTAB: DFB 40H,73H,77H,6DH,6DH,40H  ; -,P,A,S,S,-
UPATAB: DFB 3EH,54H,73H,77H,6DH,6DH  ; U,n,P,A,S,S


        ;****************************
        ;*** 7-SEGMENT CODE TABLE ***
        ;****************************

SEGTAB: DFB  3FH,06H,5BH,4FH,66H   ; 0 1 2 3 4
        DFB  6DH,7DH,07H,7FH,6FH   ; 5 6 7 8 9


        ;****************************
        ;*** VOLTAGE OUTPUT TABLE ***
        ;****************************

OUTTAB: DFB  000H,005H,00AH,00FH,014H
        DFB  019H,01FH,024H,029H,02EH
        DFB  033H,038H,03DH,042H,047H
        DFB  04DH,052H,057H,05CH,061H
        DFB  066H,06BH,070H,075H,07AH
        DFB  07FH,085H,08AH,08FH,094H
        DFB  099H,09EH,0A3H,0A8H,0AEH
        DFB  0B3H,0B8H,0BDH,0C2H,0C7H
        DFB  0CCH,0D1H,0D6H,0DBH,0E1H
        DFB  0E6H,0EBH,0F0H,0F5H,0FAH
        DFB  0FFH

OLDV:   DFS  1

        END
